#
# Succession rules - SDB format
#
#
# This file describes a simple hierarchical rule base for
# describing forest succession events.  The clauses are
# checked and when a match occurs the fields in the database
# are updated accordingly.
#
# The succession events described in this database are primarily
# old-growth decay and renewal.  Only a few of the major working
# groups have been described (spruce, poplar).  The general
# pattern is that spruce on poor sites will remain in a stable,
# old condition, spruce on medium sites will revert to younger
# spruce, and on good sites poplar will invade if it is present.
#
# Note that the last clause of every expression is either a
# 1 or 0.  This is so evaluation of the expression will return
# '1' if the stand underwent a succession event, or '0' if
# not.  This can be used with a 'select' statement to obtain a
# grouping of modified stands, e.g.:
#     $db select and $NaturalSuccession
#
set NaturalSuccession \
    {Wg?(==,"B",SITE?(==,"1",age?(>,80,
                    (YR_ORG=date-20;Stk_B=1.0;1)):0)
        :(==,"2",age?(>,80,
                    (YR_ORG=date-20;Stk_B=1.0;1)):0)
        :(==,"3",age?(>,80,
                    (YR_ORG=date-20;Stk_B=1.0;1)):0)
        :(==,"X",age?(>,80,
                    (SITE="1";YR_ORG=date-20;Stk_B=1.0;1)):0):0)
    :(==,"PJ",SITE?(==,"1",age?(>,110,
                    (YR_ORG=date-15;Stk_PJ=1.0;1)):0)
        :(==,"2",age?(>,110,
                    (YR_ORG=date-15;Stk_PJ=1.0;1)):0)
        :(==,"3",age?(>,110,
                    (YR_ORG=date-15;Stk_PJ=1.0;1)):0)
        :(==,"X",age?(>,110,
                    (YR_ORG=date-15;Stk_PJ=1.0;1)):0):0)
    :(==,"PO",SITE?(==,"1",age?(>,100,
                    (YR_ORG=date-15;Stk_PO=1.0;1)):0)
        :(==,"2",age?(>,100,
                    (YR_ORG=date-15;Stk_PO=1.0;1)):0)
        :(==,"3",age?(>,100,
                    (YR_ORG=date-15;Stk_PO=1.0;1)):0)
        :(==,"X",age?(>,100,
                    (YR_ORG=date-15;Stk_PO=1.0;1)):0):0)
    :(==,"SB",SITE?(==,"1",age?(>,170,Stk_PO?(==,0,
                    (YR_ORG=date-20;Stk_SB=1.0;1))
                :(<,0.3,
                    (Wg="PO";YR_ORG=date-30;Stk_PO=0.6;Stk_SB=0.5;1))
                :(Wg="PO";YR_ORG=date-40;Stk_PO=0.8;Stk_SB=0.3;1)):0)
        :(==,"2",age?(>,190,Stk_PO?(==,0,
                    (YR_ORG=date-55;Stk_SB=1.0;1))
                :(YR_ORG=date-50;Stk_SB=0.7;Stk_PO=0.5;1)):0)
        :(==,"3",age?(>,170,
                    (YR_ORG=date-180;Stk_SB=1.0;1)):0)
        :(==,"X",age?(>,120,Stk_PO?(==,0,
                    (YR_ORG=date-20;Stk_SB=0.8;1))
                :(<,0.3,
                    (YR_ORG=date-10;Stk_SB=0.3;Stk_PO=0.7;1))
                :(Wg="PO";SITE="1";YR_ORG=date-15;Stk_PO=0.8;Stk_SB=0.2;1)):0):0)
    :(==,"SW",SITE?(==,"1",age?(>,170,Stk_PO?(==,0,
                    (YR_ORG=date-20;Stk_SW=0.8;1))
                :(<,0.3,
                    (Wg="PO";YR_ORG=date-30;Stk_PO=0.6;Stk_SW=0.5;1))
                :((Wg="PO";YR_ORG=date-40;Stk_PO=0.8;Stk_SW=0.3;1))):0)
        :(==,"2",age?(>,190,Stk_PO?(==,0,
                    (YR_ORG=date-55;Stk_SW=1.0;1))
                :(YR_ORG=date-50;Stk_SW=0.7;Stk_PO=0.5;1)):0)
        :(==,"3",age?(>,170,
                    (YR_ORG=date-180;1)):0)
        :(==,"X",age?(>,120,Stk_PO?(==,0,
                    (YR_ORG=date-20;Stk_SW=0.8;1))
                :(<,0.3,
                    (YR_ORG=date-10;Stk_SW=0.3;Stk_PO=0.7;1))
                :(Wg="PO";SITE="1";YR_ORG=date-15;Stk_PO=0.8;Stk_SW=0.2;1)):0):0):0}

$db select set
$db select and $NaturalSuccession