#
# Script to load up the Prince Albert Model Forest Dataset
#
# Tom Moore, Oct 25, 1995
#

#
# Check to see if the database has been opened yet.  If not, open
# it now.
#
set OK 0
if {[info exists db]} {
    if [info exist $db] then {
	global $db
	set OK 1
    }
}
if {!$OK} {
    global pamf
    set db [sdb open pamf pamf.inv]
}

#
# Convert the DATE character column into a numeric constant
#
$db define date [set [set db](DATE)]
$db delete DATE

#
# Convert the ORIGIN character column into a numeric column
#
$db allocate origin NUMBER
$db calculate {origin=atof(ORIGIN)}
$db delete ORIGIN

#
# Set up a dynamic age-class field
#
$db define Age {date-origin}
$db delete AGE_CLASS
$db define age_class {Age?(<,0,"error"):(<,20,"0-19"):(<,40,"20-39"):(<,60,"40-59"):(<,80,"60-79"):(<,100,"80-99"):(<,120,"100-119"):(<,140,"120-139"):"140+"}

#
# Load in the yield curve set
#
$db curve load normal normal.yld

#
# Set up dynamic age fields in each of the sub-components
#
for {set i 1} {$i < 7} {incr i} {
    $db delete VOLUME_[set i]
    $db allocate Origin_[set i] NUMBER
    $db calculate "Origin_[set i] = date - AGE_[set i]"
    $db delete AGE_[set i]
    $db define Age_[set i] "date - Origin_[set i]"
}

#
# Define the new volume fields to be based on the curves
#
$db define Volume_1 {interpolate("normal" (CODE=CODE_1; SITE=SITE_1),
                                  "Age","Volume",Age_1) * STKG_1}
$db define Volume_2 {interpolate("normal" (CODE=CODE_2; SITE=SITE_2),
                                  "Age","Volume",Age_2) * STKG_2}
$db define Volume_3 {interpolate("normal" (CODE=CODE_3; SITE=SITE_3),
                                  "Age","Volume",Age_3) * STKG_3}
$db define Volume_4 {interpolate("normal" (CODE=CODE_4; SITE=SITE_4),
                                  "Age","Volume",Age_4) * STKG_4}
$db define Volume_5 {interpolate("normal" (CODE=CODE_5; SITE=SITE_5),
                                  "Age","Volume",Age_5) * STKG_5}
$db define Volume_6 {interpolate("normal" (CODE=CODE_6; SITE=SITE_6),
                                  "Age","Volume",Age_6) * STKG_6}

#
# Define a field that sums all the volume in the stand
#
$db define Volume {Volume_1 + Volume_2 + Volume_3 + 
                    Volume_4 + Volume_5 + Volume_6}

#
# Define a field that computes total stand volume
#
$db define Total_Volume {Volume * AREA}

#
# Load in the lookup table
#
$db join load extra multispp.rul

#
# Define the succession rules
#
source succession.tcl

#
# The model is all set to go.  The age of the inventory can be advanced
# by changing the 'date' variable, e.g.:  
#    set [set db](date) 2010
#
# Apply the natural succession rules with a calculate command:
#     $db calculate $NaturalSuccession
# or better yet, use a select command so that records that undergo a
# transition become the selected set:
#     $db select and $NaturalSuccession
#
# Generate queries using the 'summary' command:
#    $db summary CODE SITE AREA Volume
#
# Build HSI models using the 'define' command or with a 'join' command.
#