#include "sdbInt.h"

typedef struct SdbSortList {
    SdbVariable            *id;
    int                     order,
                            transient;
    struct SdbSortList     *next;
} SdbSortList;

static SdbSortList     *sort_item;
static SdbFile         *sort_dbPtr;
static SdbVariable     *sort_vars;
static int             *sort_base;
static void             list_sort _ANSI_ARGS_((int left, int right));
static int              list_compare _ANSI_ARGS_((int left, int right));
static int              list_compare_helper _ANSI_ARGS_((int left, int right, 
				  SdbSortList *item));
int
SdbSort(interp, dbPtr, argc, argv)
     Tcl_Interp             *interp;
     SdbFile                *dbPtr;
     int                     argc;
     char                  **argv;
{

    int                     i,
                            ascending,
                            prefix,
                            result;
    SdbVariable            *vars,
                           *temp;

    SdbSortList           **item,
                           *first = NULL,
                           *next;

    item = &first;

    if (argc==0) {
      Tcl_AppendResult(interp, "wrong number of arguments",NULL);
      return TCL_ERROR;
    }

    for (i = 0; i < argc; i++) {

        ascending = 1;
        prefix = 0;
        if (argv[i][0] == '+') {
            prefix = 1;
        } else if (argv[i][0] == '-') {
            ascending = 0;
            prefix = 1;
        }
        vars = SdbFound(dbPtr->varPtr, (char *) &argv[i][prefix]);
        if (!vars) {
            Tcl_AppendResult(interp, "variable not found \"", &argv[i][prefix],
                    "\"", NULL);
            result = TCL_ERROR;
            goto clean_up;
        }
        *item = (SdbSortList *) ckalloc(sizeof(SdbSortList));

        (*item)->order = ascending;

        /* if this is an expression or a lookup field, then 'calc' it 
         * into a transient field, and do the sort on the transient.
         * This should improve performance significantly for some 
         * expressions. */
        if (vars->flag & (SDB_EXPRESSION | SDB_LOOKUP)) {
            char                    unique_name[256];

            sprintf(unique_name, "%s_tmp_alloc", vars->id);
            temp = SdbInsert(dbPtr, unique_name,
                    SDB_ALLOCATED |
                    (vars->flag & (SDB_CODE | SDB_NUMBER)),
                    dbPtr->numRec);
            sprintf(unique_name, "%s_tmp_alloc=%s", vars->id, vars->id);
            SdbCalculate(interp, dbPtr, unique_name);
            (*item)->id = temp;
            (*item)->transient = 1;
        } else {
            (*item)->id = vars;
            (*item)->transient = 0;
        }
        (*item)->next = NULL;
        item = &(*item)->next;

    }

    /* set up the globals and call the qsort routine */
    sort_item = first;
    sort_dbPtr = dbPtr;
    sort_vars = vars;
    sort_base = dbPtr->base;
    list_sort(0, dbPtr->numRec - 1);

    /* set up a list that contains reverse pointers (given an actual
     * record number, what is the virtual position in the sorted list) */
    for (i = 0; i < dbPtr->numRec; i++) {
        dbPtr->reverse[dbPtr->base[i]] = i;
    }
    Tcl_ResetResult(interp);
    result = TCL_OK;

    /* Remove the linked list of sort fields.  Delete transient fields */
  clean_up:
    next = first;
    while (next) {
        first = first->next;
        if (next->transient)
            SdbDelete(interp, dbPtr, (next->id)->id,0);
        free(next);
        next = first;
    }

    return result;
}


static int
list_compare_helper(left, right, item)
     int                     left;
     int                     right;
     SdbSortList            *item;
{

    int                     val;
    SdbVariable            *vars = item->id;

    /* if CODE do a string compare */
    if (vars->flag & SDB_CODE) {
        char                   *code1,
                               *code2;

        sort_dbPtr->stamp++;
        code1 = SdbGetCode(sort_dbPtr, vars, left);
        sort_dbPtr->stamp++;
        code2 = SdbGetCode(sort_dbPtr, vars, right);
        val = strcmp(code1, code2);
    }
    /* if NUMBER do a double compare */
    else {
        double                  dval,
                                val1,
                                val2;

        sort_dbPtr->stamp++;
        val1 = SdbGetNumber(sort_dbPtr, vars, left);
        sort_dbPtr->stamp++;
        val2 = SdbGetNumber(sort_dbPtr, vars, right);
        dval = val1 - val2;
        if (dval < 0)
            val = -1;
        else if (dval > 0)
            val = 1;
        else
            val = 0;
    }

    /* toggle if not ascending order */
    if (!item->order)
        val *= -1;

    /* break ties using the next sort item in the list */
    if ((val == 0) && (item->next))
        val = list_compare_helper(left, right, item->next);

    return val;
}

static int
list_compare(left, right)
     int                     left;
     int                     right;
{

    return list_compare_helper(left, right, sort_item);

}

static void
list_sort(left, right)
     int                     left;
     int                     right;
{
    int                     i,
                            j;
    static int              k;
    int                     temp,
                            middle;

    i = left;
    j = right;
    k = (i + j) / 2;

    middle = sort_base[k];

    do {
        while ((list_compare(sort_base[i], middle) < 0) && (i < right)) {
            i++;
        }
        while ((list_compare(middle, sort_base[j]) < 0) && (j > left)) {
            j--;
        }

        if (i <= j) {
            temp = sort_base[i];
            sort_base[i] = sort_base[j];
            sort_base[j] = temp;
            i++;
            j--;
        }
    } while (i <= j);

    if (left < j)
        list_sort(left, j);
    if (i < right)
        list_sort(i, right);
}
